package com.bachelortask.warehouseapp;

import com.bachelortask.warehouseapp.category.Category;
import com.bachelortask.warehouseapp.category.CategoryRepository;
import com.bachelortask.warehouseapp.fakedata.ProductManager;
import com.bachelortask.warehouseapp.fakedata.ReviewMaker;
import com.bachelortask.warehouseapp.fakedata.UserFactory;
import com.bachelortask.warehouseapp.order.OrderRepository;
import com.bachelortask.warehouseapp.order.Orders;
import com.bachelortask.warehouseapp.product.Product;
import com.bachelortask.warehouseapp.review.ReviewRepository;
import com.bachelortask.warehouseapp.user.Customer;
import com.bachelortask.warehouseapp.user.CustomerRepository;
import com.bachelortask.warehouseapp.warehouse.Warehouse;
import com.bachelortask.warehouseapp.warehouse.WarehouseRepository;
import com.github.javafaker.Faker;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.security.servlet.SecurityAutoConfiguration;
import org.springframework.context.annotation.Bean;
import com.bachelortask.warehouseapp.product.ProductRepository;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * Warehouse application.
 * Main application of bachelor thesis in data engineering at NTNU Ålesund May 2021
 * @author Sigurdur Hallur Jonsson
 * @autonr Sigurd Strøm
 * @author Samuel Hardaberg
 */
@SpringBootApplication(exclude = SecurityAutoConfiguration.class)
public class WarehouseappApplication {

    public static void main(String[] args) {

        SpringApplication.run(WarehouseappApplication.class, args);
    }

    /**
     * CORS configuration allowing localhost on port 8080 to connect securely to this application.
     * @return
     */
    @Bean
    public WebMvcConfigurer corsConfigurer() {
        return new WebMvcConfigurer() {
            @Override
            public void addCorsMappings(CorsRegistry registry) {
                registry.addMapping("/products").allowedOrigins("http://localhost:8080");
            }
        };
    }

    /**
     * Command line runner that generates testing data for the application
     * @param productRepository
     * @param categoryRepository
     * @param reviewRepository
     * @param customerRepository
     * @param orderRepository
     * @param warehouseRepository
     * @return
     */
    @Bean
    CommandLineRunner clr(ProductRepository productRepository,
                          CategoryRepository categoryRepository,
                          ReviewRepository reviewRepository,
                          CustomerRepository customerRepository,
                          OrderRepository orderRepository,
                          WarehouseRepository warehouseRepository) {
        return args -> {

            Faker faker = new Faker();

            // WAREHOUSES
            Warehouse aes = new Warehouse("Aalesund", 8091);
            Warehouse trd = new Warehouse("Trondheim", 8092);
            Warehouse osl = new Warehouse("Oslo", 8093);
            Warehouse[] listOfWarehouses = {aes, trd, osl};

            warehouseRepository.saveAll(List.of(aes, trd, osl));

            // CATEGORIES
            Category computer = new Category("computers");
            Category tv = new Category("tvs");
            Category camera = new Category("cameras");
            Category book = new Category("books");

            categoryRepository.saveAll(List.of(computer, tv, camera, book));


            // PRODUCTS
            ProductManager productManager = new ProductManager();

            ArrayList<Product> products = new ArrayList<>();

            products.addAll(productManager.getProductComputer(50));
            products.addAll(productManager.getProductBook(50));
            products.addAll(productManager.getProductCamera(50));
            products.addAll(productManager.getProductTV(50));

            productRepository.saveAll(products);

            // REVIEWS
            try {
                ReviewMaker reviewMaker = new ReviewMaker();

                for (Product p : products) {
                    int reviews = faker.random().nextInt(0, 7);
                    for (int i = 0; i < reviews; i++) {
                        reviewRepository.save(reviewMaker.makeReview(p));
                    }
                }
            } catch (Exception e){
                System.out.println("Review reviews");
            }


            // USERS
            ArrayList<Customer> users = new ArrayList<>();
            try {
                UserFactory uf = new UserFactory();
                users.addAll(uf.makeUsers(50));

                customerRepository.saveAll(users);
            } catch(Exception e){
                System.out.println("Something wong with users");
            }

            // ORDERS
            try {
                ArrayList<Orders> ordersList = new ArrayList<>();

                for (Customer customer : users) {
                    int purchases = faker.random().nextInt(0, 5);
                    int days = faker.random().nextInt(0, 1520);


                    for (int i = 0; i < purchases; i++) {
                        int randWarehouse = faker.random().nextInt(1, listOfWarehouses.length);
                        int quantity = faker.random().nextInt(1,4);
                        double price = faker.random().nextInt(150, 25000);
                        int productNumber = faker.random().nextInt(1, products.size() - 1);
                        ordersList.add(new Orders(LocalDateTime.now().minusDays(days),
                                customer,
                                products.get(productNumber),
                                listOfWarehouses[randWarehouse - 1], quantity, price));
                    }
                }

                orderRepository.saveAll(ordersList);
            } catch(Exception e){
                System.out.println("ORDERS not in order");
                System.out.println(e.getMessage());

            }





        }; // END OF CommandLineRunner
    }


}
